package xyz.lychee.lagfixer.libs.adventure.title;

import java.time.Duration;
import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.UnknownNullability;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class TitleImpl implements Title {
   private final Component title;
   private final Component subtitle;
   @Nullable
   private final Title.@Nullable Times times;

   TitleImpl(final @NotNull Component title, final @NotNull Component subtitle, @Nullable final Title.@Nullable Times times) {
      this.title = (Component)Objects.requireNonNull(title, "title");
      this.subtitle = (Component)Objects.requireNonNull(subtitle, "subtitle");
      this.times = times;
   }

   public @NotNull Component title() {
      return this.title;
   }

   public @NotNull Component subtitle() {
      return this.subtitle;
   }

   @Nullable
   public Title.@Nullable Times times() {
      return this.times;
   }

   public <T> @UnknownNullability T part(final @NotNull TitlePart<T> part) {
      Objects.requireNonNull(part, "part");
      if (part == TitlePart.TITLE) {
         return this.title;
      } else if (part == TitlePart.SUBTITLE) {
         return this.subtitle;
      } else if (part == TitlePart.TIMES) {
         return this.times;
      } else {
         throw new IllegalArgumentException("Don't know what " + part + " is.");
      }
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         TitleImpl that = (TitleImpl)other;
         return this.title.equals(that.title) && this.subtitle.equals(that.subtitle) && Objects.equals(this.times, that.times);
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = this.title.hashCode();
      result = 31 * result + this.subtitle.hashCode();
      result = 31 * result + Objects.hashCode(this.times);
      return result;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("title", this.title), ExaminableProperty.of("subtitle", this.subtitle), ExaminableProperty.of("times", this.times));
   }

   public String toString() {
      return Internals.toString(this);
   }

   static class TimesImpl implements Title.Times {
      private final Duration fadeIn;
      private final Duration stay;
      private final Duration fadeOut;

      TimesImpl(final @NotNull Duration fadeIn, final @NotNull Duration stay, final @NotNull Duration fadeOut) {
         this.fadeIn = (Duration)Objects.requireNonNull(fadeIn, "fadeIn");
         this.stay = (Duration)Objects.requireNonNull(stay, "stay");
         this.fadeOut = (Duration)Objects.requireNonNull(fadeOut, "fadeOut");
      }

      public @NotNull Duration fadeIn() {
         return this.fadeIn;
      }

      public @NotNull Duration stay() {
         return this.stay;
      }

      public @NotNull Duration fadeOut() {
         return this.fadeOut;
      }

      public boolean equals(final @Nullable Object other) {
         if (this == other) {
            return true;
         } else if (!(other instanceof TimesImpl)) {
            return false;
         } else {
            TimesImpl that = (TimesImpl)other;
            return this.fadeIn.equals(that.fadeIn) && this.stay.equals(that.stay) && this.fadeOut.equals(that.fadeOut);
         }
      }

      public int hashCode() {
         int result = this.fadeIn.hashCode();
         result = 31 * result + this.stay.hashCode();
         result = 31 * result + this.fadeOut.hashCode();
         return result;
      }

      public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
         return Stream.of(ExaminableProperty.of("fadeIn", this.fadeIn), ExaminableProperty.of("stay", this.stay), ExaminableProperty.of("fadeOut", this.fadeOut));
      }

      public String toString() {
         return Internals.toString(this);
      }
   }
}
